@@ -26,6 +26,8 @@
 
 #if defined(__GNUC__) && !defined(__STRICT_ANSI__)
 
+#define _GNU_SOURCE 1
+
 #ifdef DSP_DEBUG
 #define DPRINTF(format, args...)	printf(format, ## args)
 #else
@@ -45,6 +47,7 @@
 #include <sys/time.h>
 #include <stdio.h>
 #include <errno.h>
+#include <pthread.h>
 
 #ifdef HAVE_MACHINE_SOUNDCARD_H
 #  include <machine/soundcard.h>
@@ -124,16 +127,20 @@ set_volume (int left, int right)
 }
 
 
+pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+
 static void
 dsp_init (void)
 {
+  pthread_mutex_lock(&mutex);
   if (!ident)
     {
       const char *str;
      
       str = getenv ("ESDDSP_NAME");
-      ident = malloc (ESD_NAME_MAX);
+      ident = malloc (ESD_NAME_MAX+1);
       strncpy (ident, (str ? str : "esddsp"), ESD_NAME_MAX);
+      ident[ESD_NAME_MAX] = '\0';
 
       str = getenv("ESDDSP_MMAP");
       mmapemu = str && !strcmp(str,"1");
@@ -157,6 +164,7 @@ dsp_init (void)
 	  DPRINTF ("mixer settings file: %s\n", mixer);
 	}
     }
+  pthread_mutex_unlock(&mutex);
 }
 
 static void
@@ -225,7 +233,10 @@ open_wrapper (int (*func) (const char *, int, mode_t),
   dsp_init ();
 
   va_start (args, flags);
-  mode = va_arg (args, mode_t);
+  if (sizeof (mode_t) < sizeof (int))
+	  mode = va_arg (args, int);
+  else
+	  mode = va_arg (args, mode_t);
   va_end (args);
 
   if (!strcmp (pathname, "/dev/dsp"))
@@ -676,6 +687,33 @@ mmap (void  *start,  size_t length, int prot, int flags, int fd, off_t offset)
   return (void *)-1;
 }
 
+void *
+mmap64 (void  *start,  size_t length, int prot, int flags, int fd, off64_t offset)
+{
+  static void *(*func) (void *, size_t, int, int, int, off64_t) = NULL;
+  
+  if (!func)
+    func = (void * (*) (void *, size_t, int, int, int, off64_t)) dlsym (REAL_LIBC, "mmap64");
+
+  if(fd != sndfd || sndfd == -1)
+    return (*func)(start,length,prot,flags,fd,offset);
+  else
+    {
+      DPRINTF ("esddsp: mmap64 - start = %x, length = %d, prot = %d\n",
+	       start, length, prot);
+      DPRINTF ("      flags = %d, fd = %d, offset = %lld\n",flags, fd,offset);
+      if(mmapemu)
+	{
+	  mmapemu_osize = length;
+	  mmapemu_obuffer = malloc(length);
+	  mmapemu_ocount.ptr = mmapemu_ocount.blocks = mmapemu_ocount.bytes = 0;
+	  return mmapemu_obuffer;
+	}
+      else DPRINTF ("esddsp: /dev/dsp mmap64 (unsupported, try -m option)...\n");
+    }
+  return (void *)-1;
+}
+
 int
 munmap (void *start, size_t length)
 {
